//  Main.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <stdlib.h>

#include 	"Main.h"
#include 	"MYudit.h"
#include 	"MainWidget.h"
#include	"AString.h"
#include 	"Config.h"
#include	<Dialog.h>
#include 	<iostream.h>
#include	<Translate.h>
#include	<X11/Xlib.h>
#include	<X11/Intrinsic.h>

XtAppContext	muedTopContext;
Widget		muedTopWidget;
Display*	muedDisplay;
int		muedScreen;

static	char		title[] = "Yudit";

static char *appDefaults[] = {
#include "Yudit.ad.h"
0
};

MYuditResources		muedResources;
MYuditPromptResources 	muedPromptResources;

static XtResource  muedAppResources[] =
{
	{
		"cursorColor",
		"CursorColor",
		XtRPixel,
		sizeof (Pixel),
		XtOffsetOf (MYuditResources, CursorColor),
		XtRString,
		(XtPointer) "Yellow"
	} 
};

static XtResource  muedPromptAppResources[] =
{
	{
		"intallColormap",
		"IntallColormap",
		XtRBoolean,
		sizeof (Boolean),
		XtOffsetOf (MYuditPromptResources, InstallColormap),
		XtRString,
		(XtPointer) "False" /*- colormap is used -*/
	},{
	
		"encoding",
		"Encoding",
		XtRString,
		sizeof (String),
		XtOffsetOf (MYuditPromptResources, Encoding),
		XtRString,
		(XtPointer) 0 /*- default encoding -*/
	}
};

static XrmOptionDescRec options[] = {
        { 
		"-install",     
		".intallColormap", 
		XrmoptionNoArg,  
		"True"
 	},
        { 
		"-e",     
		".encoding", 
		XrmoptionSepArg,  
		0,
 	}
};

static Config*		lConfig;

char*		xEnvironment;

int
main (int argc, char *argv[])
{
	const char* 		errors;
	Dialog*			dialog;
	const TrueTypeFont*	xTrueTypeFont;
	int			i;
	int			j;
	int			k;
	const FontMap*		fontMap;
	const char*		umap;
	const char*		foundry;
	const char*		family;
	const char*		registry;
	const char*		encoding;
	AString			errMessage;
	void*			vcludge;
	XtPointer		cludge;
	AString			xfiles;
	UFreeType*		xFreeType;
	const char*		xFileName;
	Config			config ("yuditrc");

	lConfig = &config;

	xfiles = (const char*) "XFILESEARCHPATH=";
	xfiles += (const char*) getenv ("XFILESEARCHPATH");
	xfiles += (const char*) ":";
	xfiles += (const char*) XFILESEARCHPATH;
	xEnvironment = new char [strlen ((const char*)xfiles) + 1 ];
	strcpy (xEnvironment, (const char*)xfiles);
    	putenv(xEnvironment);

#ifdef USE_SETLOCALE
	setlocale (LC_ALL, "");
	if (XSupportsLocale())
	{
		XSetLocaleModifiers ("");
		XtSetLanguageProc(NULL,(XtLanguageProc)NULL,NULL);
	}	
	InitTranslate (/* domain here */);
#endif
	vcludge = &argc;
	cludge = vcludge;
	muedTopWidget = XtVaAppInitialize (&muedTopContext, "Yudit", 
		options, XtNumber(options), &argc, argv, 
		(String*) appDefaults, XtNtitle, (XtArgVal)MYuditTitle(), NULL);


	vcludge = &muedPromptResources;
	cludge = vcludge;
	XtVaGetApplicationResources (muedTopWidget,
			cludge, muedPromptAppResources, 
			XtNumber (muedPromptAppResources), NULL);

	MYuditSetColorMap (muedTopWidget, muedPromptResources.InstallColormap);

	vcludge = &muedResources;
	cludge = vcludge;
	XtVaGetApplicationResources (muedTopWidget,
			cludge, muedAppResources,
			XtNumber (muedAppResources), NULL);

	muedDisplay = XtDisplay (muedTopWidget);
	muedScreen = XDefaultScreen (muedDisplay);

	XtVaSetValues (muedTopWidget,
		XmNx,		0,
		XmNy,		0,
		XmNwidth,	DisplayWidth (muedDisplay, muedScreen),
		XmNheight,	DisplayHeight (muedDisplay, muedScreen),
		0);


#ifdef USE_EDITRES
	XtAddEventHandler (EdoXToplevel, (EventMask)0, True,
		_XEditResCheckMessages, NULL);
#endif

	if (config.getConfigPath().isNull())
	{
		dialog = new Dialog (muedTopWidget,
				"Yudit Info", Dialog::OK, Dialog::DINFO);
		dialog->setMessage (Translate ("Copying system config file."));
		dialog->getButton ();
		delete dialog;
		config.copySysConfig("yuditrc");
		errors =  config.getErrors();
		if (errors!=0)
		{
			cerr << "error: config " << errors << "\n";
			dialog = new Dialog (muedTopWidget,
				"Yudit Error", Dialog::OK);
			dialog->setMessage (Translate (errors));
			dialog->getButton ();
			delete dialog;
			exit (1);
		}
		config.readConfig ("yuditrc");
	}

	errors = config.getErrors();
	
	if (errors!=0)
	{
		cerr << "error: config " << errors << "\n";
		dialog = new Dialog (muedTopWidget,
				"Yudit Error", Dialog::OK);
		dialog->setMessage (Translate (errors));
		dialog->getButton ();
		delete dialog;
		exit (1);
	}

	MYuditInit ();

	USetMapPath ((const char*) config.getPath ());
	//
	// Load the fontMaps
	//
	for (i=0; i<config.getCheckList (Config::FONTNAME).getSize(); i++)
	{
		fontMap = config.getFontMap (
			config.getCheckList (Config::FONTNAME).at(i));
		if (fontMap!=0) for (j=0; j<fontMap->size; j++)
		{
			foundry = fontMap->list[FontMap::FOUNDRY].at (j);
			family = fontMap->list[FontMap::FAMILY].at (j);
			registry = fontMap->list[FontMap::REGISTRY].at (j);
			encoding = fontMap->list[FontMap::ENCODING].at (j);
			umap = fontMap->list[FontMap::UMAP].at (j);
			
			UAddFontMap ((const char*) fontMap->name,
				foundry, family, registry, encoding, umap);
		}
		if (fontMap==0)
		{
			xTrueTypeFont = config.getTrueTypeFont(
				config.getCheckList (Config::FONTNAME).at(i));
			if (xTrueTypeFont!=0)
			{
				UAddFreeType ((const char*)xTrueTypeFont->name);
				xFreeType = UGetFreeTypePrint (
					(const char*)xTrueTypeFont->name);
				if (xFreeType==0)
				{
					cerr << "error: count not add TrueType font '"
					 << (const char*)xTrueTypeFont->name
					 << "'.\n";
					continue;
				}
				for (j=0; j<UFreeType::WEIGHT_COUNT; j++)
				{
				 for (k=0; k<UFreeType::SLANT_COUNT; k++)
				 {
					xFileName = (const char*)xTrueTypeFont->fontFile[j][k];
					xFreeType->setFontFile (
						xFileName, 
						(UFreeType::UWeight) j, 
						(UFreeType::USlant) k);
				 }
				}
			}
		}
	}

	if (UGetFontMap ((const char*)config.getChecked(Config::FONTNAME))==0)
	{
		errMessage = "error: config Font Map ";
		errMessage += (const char*) config.getChecked(Config::FONTNAME);
		errMessage += " is not available.";

		cerr << (const char*) errMessage;
		errMessage += "\n";

		dialog = new Dialog (muedTopWidget,
				"Yudit Error", Dialog::OK);

		dialog->setMessage ((const char*) errMessage);
		dialog->getButton ();
		delete dialog;
		exit (1);
	}

	// self destroy
	if (argc <= 1)
	{
		if (muedPromptResources.Encoding != 0)
		{
			config.setChecked (Config::ENCODING, 
					muedPromptResources.Encoding);
		}
		new MainWidget ("Yudit", muedTopWidget, config, 0);
	}
	else
	{
		for (i=1; i< argc; i++)
		{
			new MainWidget ("Yudit", muedTopWidget, config, 
					argv[i], muedPromptResources.Encoding);
		}
	}
	XtAppMainLoop (muedTopContext);
	
}

const Config&
MYuditGetConfig ()
{
	return *lConfig;
}

const MYuditResources* 
MYuditGetResources () {return &muedResources; }

const MYuditPromptResources*
MYuditGetPromptResources () {return &muedPromptResources; }

const XtAppContext 
MYuditGetTopContext () { return muedTopContext ; }

const Widget
MYuditGetTopWidget () { return muedTopWidget ; }

Display*
MYuditGetDisplay () { return muedDisplay ; }

int
MYuditGetScreen () { return muedScreen; };

char*
MYuditTitle () { return title; }
